94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
121 lines
3.8 KiB
Python
121 lines
3.8 KiB
Python
import torch
|
|
import triton
|
|
import triton.language as tl
|
|
from triton.language.extra import libdevice
|
|
|
|
softcap_out_autotune = triton.autotune(
|
|
configs=[
|
|
triton.Config(kwargs={"BLOCK_SIZE": 128}, num_warps=4),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 128}, num_warps=8),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 128}, num_warps=16),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 256}, num_warps=4),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 256}, num_warps=8),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 512}, num_warps=4),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 512}, num_warps=8),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 512}, num_warps=16),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 1024}, num_warps=4),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 1024}, num_warps=8),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 1024}, num_warps=16),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 1024}, num_warps=32),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 2048}, num_warps=32),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 4096}, num_warps=32),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 8192}, num_warps=32),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 16384}, num_warps=32),
|
|
triton.Config(kwargs={"BLOCK_SIZE": 32768}, num_warps=32),
|
|
],
|
|
key=["n_ele"],
|
|
)
|
|
|
|
|
|
@triton.jit
|
|
def softcap_out_kernel(
|
|
output_ptr,
|
|
input_ptr,
|
|
n_ele,
|
|
softcap_const: tl.constexpr,
|
|
BLOCK_SIZE: tl.constexpr,
|
|
):
|
|
pid = tl.program_id(axis=0)
|
|
block_start = pid * BLOCK_SIZE
|
|
offsets = block_start + tl.arange(0, BLOCK_SIZE)
|
|
mask = offsets < n_ele
|
|
x = tl.load(input_ptr + offsets, mask=mask)
|
|
fx = x.to(tl.float32)
|
|
fxs = fx / softcap_const
|
|
exped = tl.exp(2 * fxs)
|
|
top = exped - 1
|
|
bottom = exped + 1
|
|
output = top / bottom * softcap_const
|
|
tl.store(output_ptr + offsets, output, mask=mask)
|
|
|
|
|
|
softcap_out_kernel_autotuned = softcap_out_autotune(softcap_out_kernel)
|
|
|
|
|
|
def softcap_out(x, softcap_const, autotune=False):
|
|
output = torch.empty_like(x, dtype=torch.float32)
|
|
n_elements = output.numel()
|
|
if autotune:
|
|
|
|
def grid(meta):
|
|
return (triton.cdiv(n_elements, meta["BLOCK_SIZE"]),)
|
|
|
|
softcap_out_kernel_autotuned[grid](output, x, n_elements, softcap_const)
|
|
else:
|
|
softcap_out_kernel[(triton.cdiv(n_elements, 128),)](
|
|
output, x, n_elements, softcap_const, BLOCK_SIZE=128, num_warps=8
|
|
)
|
|
return output
|
|
|
|
|
|
@triton.jit
|
|
def softcap_inplace_logits_kernel(
|
|
full_logits_ptr,
|
|
softcapping_value,
|
|
ncols,
|
|
row_stride,
|
|
BLOCK_SIZE: tl.constexpr,
|
|
):
|
|
row = tl.program_id(1).to(tl.int64)
|
|
pid = tl.program_id(0).to(tl.int64)
|
|
block_start = pid * BLOCK_SIZE
|
|
offsets = block_start + tl.arange(0, BLOCK_SIZE)
|
|
mask = offsets < ncols
|
|
|
|
# Load values
|
|
row_ptr = full_logits_ptr + row * row_stride
|
|
x = tl.load(row_ptr + offsets, mask=mask)
|
|
|
|
# Perform operations in-place
|
|
x = x / softcapping_value
|
|
x = libdevice.tanh(x)
|
|
x = x * softcapping_value
|
|
|
|
# Store result
|
|
tl.store(row_ptr + offsets, x, mask=mask)
|
|
|
|
|
|
def softcap_inplace_logits(full_logits, final_logit_softcapping):
|
|
if full_logits.is_contiguous():
|
|
nrows, ncols = 1, full_logits.numel()
|
|
row_stride = ncols
|
|
else:
|
|
assert full_logits.ndim == 2, "non-contiguous softcap requires 2D tensor"
|
|
assert (
|
|
full_logits.stride(1) == 1
|
|
), "non-contiguous softcap requires contiguous columns"
|
|
nrows, ncols = full_logits.shape
|
|
row_stride = full_logits.stride(0)
|
|
|
|
BLOCK_SIZE = 1024
|
|
grid = ((ncols + BLOCK_SIZE - 1) // BLOCK_SIZE, nrows)
|
|
|
|
softcap_inplace_logits_kernel[grid](
|
|
full_logits_ptr=full_logits,
|
|
softcapping_value=final_logit_softcapping,
|
|
ncols=ncols,
|
|
row_stride=row_stride,
|
|
BLOCK_SIZE=BLOCK_SIZE,
|
|
)
|
|
return full_logits
|